#!/bin/bash
# This script is used for maintance cron jobs on edge-db PostgreSQL cluster
_usage() {
    echo "Usage: $0 -m info|clean|restat [-d dbname] [-u username] [-l logdir]"
    echo " NB! For default values see config/*.config!"
    echo "   -C, --config         Configuration name"
    echo "   -m, --mode           Mainteanance mode, available modes: info|clean"
    echo "   -d, --database       PostgreSQL database name"
    echo "   -u, --user           PostgreSQL user name"
    echo "   -p, --password       PostgreSQL user password"
    echo "   -P, --port           PostgreSQL port"
    echo "   -h, --host           PostgreSQL host"
    echo "   -D, --data-dir       PostgreSQL data directory"
    echo "   -l, --logs-dir       PosthreSQL logs directory"
    echo "   -?, --help           This help text"
    exit 0
}

# Must be root to run this script
if test "$EUID" -ne 0 ; then
    echo "Error: Please run as root!"
    exit -1
fi

# Current dir
_ME=$(readlink -f "$0")
PWD=$(dirname "${_ME}")
LOCK=$PWD/maintenance.lock

# Config name
[ -f ${PWD}/.configure ] && CONFIG=$(cat ${PWD}/.configure | tr -d "\n\r")
CONFIG=${CONFIG:-default}

[ -f $LOCK ] && (echo "Lock file found. Exiting."; exit -1)

# Selected mode
MMODE=

# CLI params
while test $# -gt 0; do
    case $1 in
        --config|-C)
            CONFIG="${2}"
            shift
        ;;
        --mode|-m)
            MMODE="${2}"
            shift
        ;;
        --database|-d)
            PGDBNAME="${2}"
            shift
        ;;
        --user|-u)
            PGUSER="${2}"
            shift
        ;;
        --password|-p)
            PGUSERPASS="${2}"
            shift
        ;;
        --port|-P)
            PGCLUSTERPORT="${2}"
            shift
        ;;
        --host|-h)
            PGCLUSTERHOST="${2}"
            shift
        ;;
        --data-dir|-D)
            PGDATADIR="${2}"
            shift
        ;;
        --logs-dir|-l)
            PGLOGDIR="${2}"
            shift
        ;;
        --help|-?)
            _usage
            break
        ;;
        --)
            break
        ;;
        --*|-?)
            echo "Unknown option ${1}"
            exit 11
        ;;
        -*)
            split=$1
            shift
            set -- $(echo "$split" | cut -c 2- | sed 's/./-& /g') "$@"
            continue
        ;;
        *)
            echo "Unknown argument ${1}"
            exit 11
        ;;
    esac

    shift
done

# Configs
source $PWD/config/$CONFIG.config

# Main subroutines
_maintenance_info() {
    if [ ! -d ${PGLOGDIR} ]; then mkdir -p ${PGLOGDIR}; fi
    psql -h $PGCLUSTERHOST -p $PGCLUSTERPORT -q -t -c "select * from service.maintenance_info();" ${PGDBNAME} ${PGUSER} > ${PGLOGDIR}/${PGDBNAME}.maintenance.info 2>&1
}

_maintenance_clean() {
    psql -h $PGCLUSTERHOST -p $PGCLUSTERPORT -q -t -c  "select service.maintenance_clean();" ${PGDBNAME} ${PGUSER} | logger 2>&1
}


if [ "${MMODE}" == "info" ]; then _maintenance_info; exit 0; fi
if [ "${MMODE}" == "clean" ]; then _maintenance_clean; exit 0; fi

# If unknown mode is provided or no mode at all - echo usage
_usage




