--

create or replace function edge.plate_motion_set(
    iorigin   int,
    idetector int,
    iplate    varchar,
    ilog      jsonb,
    imints    int8,
    imaxts    int8
) returns void as $$
begin
    if iorigin isnull then
        raise exception 'Invalid origin' using hint='400';
    end if;
    if iplate isnull or iplate = '' then
        raise exception 'Invalid plate' using hint='400';
    end if;
    insert into edge.plate_motions     
        (origin,detector,plate,during,log)
    values (
        iorigin,
        idetector,
        iplate,
        int8range(imints,imaxts),
        ilog
    ) on conflict (origin,detector,plate)
    do update set
        ctime = now(),
        during = int8range(imints,imaxts),
        log = ilog
    where 
        edge.plate_motions.origin = iorigin and
        edge.plate_motions.detector = idetector and
        edge.plate_motions.plate = iplate;
end;
$$ language plpgsql;

--

create or replace function edge.plate_motion_del(
    iorigin      int,
    idetector    int,
    iplate       varchar
) returns int as $$
declare
    _cnt int;
begin
    if iorigin isnull or idetector isnull then
        raise exception 'Invalid origin/detector' using hint='400';
    end if;
    delete from edge.plate_motions     
    where 
        origin = iorigin and
        detector = idetector and
        plate = iplate
    ;    
    get diagnostics _cnt = ROW_COUNT;
    return _cnt;
end;
$$ language plpgsql;

--

create or replace function edge.plate_motion_get(
    iorigin      int        default null,
    idetector    int        default null,
    iplate       varchar    default null,
    idistance    int        default null,
    imints       int8       default null,
    imaxts       int8       default null,
    out origin   int,
    out detector int,
    out plate    varchar,
    out distance int,
    out log      jsonb
) returns setof record as $$
begin
return query select 
    pm.origin,pm.detector,pm.plate,
    case when iplate isnull then 0 else
        lib.ilevenstein(pm.plate,iplate)
    end as distance,
    pm.log
from edge.plate_motions pm
where
    case when iorigin isnull then true else pm.origin = iorigin end and
    case when idetector isnull then true else pm.detector = idetector end and
    case when iplate isnull or iplate = '' then true else
        case when idistance isnull or idistance = 0 then 
            pm.plate = iplate 
        else
            lib.ilevenstein(pm.plate,iplate) <= idistance
        end    
    end and
    pm.during && int8range(imints,imaxts)
order by distance;    
end;
$$ language plpgsql;

--

create or replace function edge.plate_motion_clear(
    iorigin    int,
    idetector   int,
    imaxts      int8
) returns int4 as $$    
declare
    _cnt integer;
begin 
    delete from edge.plate_motions
    where 
        case when iorigin isnull then true else origin = iorigin end
        and 
        case when idetector isnull then true else detector = idetector end
        and 
        case when imaxts isnull then true else upper(during) < imaxts end
    ;
    get diagnostics _cnt = ROW_COUNT;
    return _cnt;
end;
$$ language plpgsql;


