
--
-- report handler on (plate.scurry)
-- PARAM:
--      origin   uuid               - null for any
--      fromtime timestamp          - time domain start (def: -infinit)
--      totime   timestamp          - time domain end (def: infinit)
--      mingap   int                - gap in seconds between separate detections (def: 10)
--      mindet   int                - minimal number of detections to be reported (def: 2)
-- RETURNS: 
--      errcode int (def:0)
--      errmsg  varchar (def:Ok)
--      result jsonb:
--          [{"total-count","plate-text","last-guid","last-datetime","last-ts"}] - frequency
--          analysis of plate detections, sorted by scurry frequency
--

create or replace function public.report_plate_scurry( iparam jsonb, out errcode integer, out errmsg varchar, out result jsonb ) as $$
declare
    iorigin   uuid;
    ifromtime timestamptz;
    itotime   timestamptz;
    imingap   int;
    imindet   int;
begin
    iorigin := (iparam ->> 'origin')::uuid;
    ifromtime := coalesce((iparam ->> 'fromtime')::timestamptz, '-infinity'::timestamptz);
    itotime   := coalesce((iparam ->> 'totime')::timestamptz,    'infinity'::timestamptz);
    imingap := coalesce((iparam ->> 'mingap')::integer, 10);
    imindet := coalesce((iparam ->> 'mindet')::integer, 2);
    select into result json_agg(r)::jsonb from (
        select plate AS "plate-text",
            count AS "total-count",
            lastguid AS "last-guid",
            lasttime AS "last-datetime"
        from (
            select distinct
                q2.plate as plate,
                count(plate) over (partition by plate) as count,
                first_value(ctime) over (partition by plate) as lasttime,
                first_value(guid) over (partition by plate) as lastguid
            from (
                select
                    q1.*,
                    not (plate = n2 and t2 notnull and abs(extract(epoch from t2-ctime)) < imingap) as outgap
                from (
                    select
                        guid,
                        ctime,
                        plate,
                        lag(ctime,1) over(order by plate,ctime desc) as t2,
                        lag(plate,1) over(order by plate,ctime desc) as n2
                    from eventindex
                    where true
                        and (iorigin isnull or (iorigin notnull and origin = iorigin))
                        and ctime > ifromtime and ctime < itotime
                    order by plate,ctime desc
                ) q1
             ) q2
            where outgap
        ) q3
        where count >= imindet
        order by count desc
    ) r;
    if result isnull then
        result := '[]'::jsonb;
    end if;    
    errcode := 0;
    errmsg := '';
exception when others then
    errcode := -12;
    errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;


--
-- report handler on (plate.traffic.combined)
-- PARAM:
--    ordinate          varchar as (none, speed, alarm, category, weight)
--    datetime-min      timestamp (null)
--    datetime-max      timestamp (null)
--    datetime-step     varchar as {hour, day, week, month, year}
--    origin            uuid (null)
--    type              varchar[] (null)
--    lane              varchar[] (null)
--    lane-stat         varchar as (undefined, defined)
--    plate-stat        varchar as (undefined, defined)
--    speed-stat        varchar as (undefined, defined)
--    speed-min         int (null)    
--    speed-max         int (null)
--    speed-step        int (null)
--    weight-min        int (null)
--    weight-max        int (null)
--    weight-step       int (null)
--    category          varchar[] as {}
--RETURNS:
--    {"limits":, "count": , "items": [ {"datetime":, "count":, "items": [ {"$ordinate":, "count":}, ] }, ]}
--
--

create or replace function public.report_traffic_combined( iparam jsonb, out errcode integer, out errmsg varchar, out result jsonb ) as $$
declare
    iordinate varchar;
    ifromtime timestamptz;
    itotime timestamptz;
    itimedelta varchar(5);
    iorigin uuid;
    ilane text[];
    itype text[];
    ifromspeed int;
    itospeed int;
    ispeeddelta int;
    ioverspeed int;
    ifromweight int;
    itoweight int;
    iweightdelta int;
    ialarms text;   
    icategory text;
    iwithlane boolean;
    iwithplate boolean;
    iwithspeed boolean;
    _str text;
    _items jsonb;
    _total int;
begin
    iordinate := lower((iparam ->> 'ordinate'));
    if iordinate isnull or not iordinate = any(array['none', 'speed','type','weight','category']) then
        iordinate := 'none';
    end if;    
    ifromtime := (iparam ->> 'datetime-min')::timestamptz;
    itotime   := (iparam ->> 'datetime-max')::timestamptz;
    itimedelta := lower((iparam ->> 'datetime-step'));
    if itimedelta isnull or not itimedelta = any(array['none', 'hour', 'day', 'week', 'month', 'year']) then
        if itotime notnull and ifromtime notnull and ((itotime - ifromtime) < interval '1 day') then
            itimedelta = 'hour';
        else
            itimedelta = 'day';
        end if;    
    end if;
    iorigin := (iparam ->> 'origin')::uuid;
    ilane :=  array(select jsonb_array_elements_text((iparam -> 'lane')));
    itype :=  array(select jsonb_array_elements_text((iparam -> 'type')));
    if cardinality(ilane) < 1 then
        ilane := null;
    end if;    
    if cardinality(itype) < 1 then
        itype := null;
    end if;    
    ifromspeed := (iparam ->> 'speed-min')::int;
    itospeed := (iparam ->> 'speed-max')::int;
    ioverspeed := (iparam ->> 'overspeed-min')::int;
    ispeeddelta := coalesce((iparam ->> 'speed-step')::int,20);
    ifromweight := (iparam ->> 'weight-min')::int;
    itoweight := (iparam ->> 'weight-max')::int;
    iweightdelta := coalesce((iparam ->> 'weight-step')::int,20);
    _str := lower((iparam ->> 'plate-stat'));
    if _str notnull and _str = any(array['defined','undefined']) then
        iwithplate := (_str = 'defined');
    end if;    
    _str := lower((iparam ->> 'speed-stat'));
    if _str notnull and _str = any(array['defined','undefined']) then
        iwithspeed := (_str = 'defined');
    end if;    
    _str := lower((iparam ->> 'lane-stat'));
    if _str notnull and _str = any(array['defined','undefined']) then
        iwithlane := (_str = 'defined');
    end if;    
    icategory := null;
    -- simple hystogramm
    if iordinate = 'none' then
        execute 
        format ( '
            select
                json_agg(r)::jsonb, sum(r.count) 
            from (    
                select 
                    case when ''%s'' = ''none'' then null else date_trunc(''%s'', ctime) end as datetime, count(*) as count 
                from 
                    eventindex 
                where true 
                    %s
                    %s
                    %s
                    %s
                    %s
                    %s
                    %s
                    %s
                    %s
                    %s
                 group by datetime
                 order by datetime
             ) r    
            ', 
            itimedelta,itimedelta,
            case when ifromtime notnull then format('and ctime >= ''%s''', ifromtime) else null end,
            case when itotime notnull then format('and ctime <= ''%s''', itotime) else null end,
            case when iorigin notnull then format('and origin = ''%s''', iorigin) else null end,
            case when ilane notnull then format('and lane = any(''%s''::text[])', ilane) else null end,
            case when itype notnull then format('and doctype = any(''%s''::text[])', itype) else null end,
            case when iwithlane notnull then format('and lane is %s null',case when iwithlane then 'not' else null end) else null end,
            case when iwithplate notnull then format('and plate is %s null',case when iwithplate then 'not' else null end) else null end,
            case when iwithspeed notnull then format('and speed is %s null',case when iwithspeed then 'not' else null end) else null end,
            case when ifromspeed notnull then format('and speed >= %s', ifromspeed) else null end,
            case when itospeed notnull then format('and speed <= %s', itospeed) else null end
            )
        into _items, _total;    
    elsif iordinate = 'speed' then
        execute
        format ( '
            select
                json_agg(r)::jsonb, sum(r.count) 
            from (   
                select
                    time as datetime, 
                    sum(count) as count, 
                    json_agg(json_build_object(''speed'',rr.speed,''count'',rr.count)) as items
                from (    
                    select 
                        case when ''%s'' = ''none'' then null else date_trunc(''%s'', ctime) end as time, (speed/%s)*%s as speed, count(*) as count 
                    from 
                        eventindex 
                    where true
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                     group by time, speed
                     order by time, speed
                 ) rr
                 group by time
                 order by time
             ) r    
            ', 
            itimedelta, itimedelta,
            ispeeddelta, ispeeddelta,
            case when ifromtime notnull then format('and ctime >= ''%s''', ifromtime) else null end,
            case when itotime notnull then format('and ctime <= ''%s''', itotime) else null end,
            case when iorigin notnull then format('and origin = ''%s''', iorigin) else null end,
            case when ilane notnull then format('and lane = any(''%s''::text[])', ilane) else null end,
            case when itype notnull then format('and doctype = any(''%s''::text[])', itype) else null end,
            case when iwithlane notnull then format('and lane is %s null',case when iwithlane then 'not' else null end) else null end,
            case when iwithplate notnull then format('and plate is %s null',case when iwithplate then 'not' else null end) else null end,
            case when iwithspeed notnull then format('and speed is %s null',case when iwithspeed then 'not' else null end) else null end,
            case when ifromspeed notnull then format('and speed >= %s', ifromspeed) else null end,
            case when itospeed notnull then format('and speed <= %s', itospeed) else null end
            )
        into _items, _total;   
    elsif iordinate = 'type' then
        execute
        format ( '
            select
                json_agg(r)::jsonb, sum(r.count) 
            from (   
                select
                    time as datetime, 
                    sum(count) as count,
                    json_agg(json_build_object(''type'',rr.type,''count'',rr.count)) as items
                from (    
                    select 
                        case when ''%s'' = ''none'' then null else date_trunc(''%s'', ctime) end as time, doctype as "type", count(*) as count 
                    from 
                        eventindex 
                    where true
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                        %s
                     group by time, "type"
                     order by time, "type"
                 ) rr
                 group by time
                 order by time
             ) r    
            ', 
            itimedelta, itimedelta,
            case when ifromtime notnull then format('and ctime >= ''%s''', ifromtime) else null end,
            case when itotime notnull then format('and ctime <= ''%s''', itotime) else null end,
            case when iorigin notnull then format('and origin = ''%s''', iorigin) else null end,
            case when ilane notnull then format('and lane = any(''%s''::text[])', ilane) else null end,
            case when itype notnull then format('and doctype = any(''%s''::text[])', itype) else null end,
            case when iwithlane notnull then format('and lane is %s null',case when iwithlane then 'not' else null end) else null end,
            case when iwithplate notnull then format('and plate is %s null',case when iwithplate then 'not' else null end) else null end,
            case when iwithspeed notnull then format('and speed is %s null',case when iwithspeed then 'not' else null end) else null end,
            case when ifromspeed notnull then format('and speed >= %s', ifromspeed) else null end,
            case when itospeed notnull then format('and speed <= %s', itospeed) else null end
            )
        into _items, _total;   
        -- TBD
    
    elsif iordinate = 'category' then

        -- TBD

    elsif iordinate = 'weight' then

        -- TBD 

    end if;

    if _items notnull and _total notnull then
        result := json_build_object('limits',iparam,'count',_total,'items',_items)::jsonb;
    else    
        result := json_build_object('limits',iparam,'count',0,'items','[]'::jsonb)::jsonb;
    end if;    

    errcode := 0;
    errmsg := '';

exception when others then
    errcode := -12;
    errmsg := format('%s: %s',SQLSTATE,to_jsonb(SQLERRM));
end;
$$ language plpgsql;


